
/*****************************************************************************************  
 *  OpenSourceSaverPrefs.m
 *  OpenSourceSaver
 *
 *  Created by Doug Diego on Thu Jun 13 2002.
 *  Copyright (c) 2002 Diego.org. All rights reserved.
 *
 *  
 *  This software is distributed under the terms of the GNU General Public License (GPL)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 *
 * THANKS for all the feed back that I have recieved.  It has been very helpful.
 ******************************************************************************************/

#import "OpenSourceSaverPrefs.h"

static NSString *DOSS = @"org.diego.OpenSourceSaver";


@implementation OpenSourceSaverPrefs

+ (void)initialize {
    if (self == [OpenSourceSaverPrefs class]) {
    }
}


static OpenSourceSaverPrefs *sharedInstance = nil;

+ (id)sharedPreferences {
    if (!sharedInstance) {
        sharedInstance = [[[self class] alloc] init];
    }
    return sharedInstance;
}


- (id)init {
    NSString *name;
    static NSMutableArray *availableFonts = nil;
    
    self = [super initWithWindowNibName:@"Preferences"];

    if (self) {
        [self loadPrefs];
        availableFonts = [[NSMutableArray allocWithZone:[self zone]] initWithArray:[[NSFontManager sharedFontManager] availableFonts]];
        name = [availableFonts objectAtIndex:1];
        font = [NSFont fontWithName:name size:10];
    }
    return self;
}

- (void)dealloc {
    if ([self isWindowLoaded]) {
        [[self window] setDelegate:nil];
    }
    [super dealloc];
}


- (void)windowDidLoad {
    NSColor *color;
    
    [super windowDidLoad];
    
    /* adjust speed of slider */
    [speedSlider setAltIncrementValue:(double)5];  
    [speedSlider setFloatValue:speed];
  
    /* set font size in text field */
    //NSLog(@"windowDidLoad() fontSize=%i", fontSize);
    [fontField setStringValue:[NSString stringWithFormat:@"%@ - %.1f", fontName, fontSize]];

    
    /* set search root in text field */
    [searchRootField setStringValue:theSearchRoot];
    
    /* set display own source checkbox */
    [ownSourceSwitch setState:displayOwnSource];
    
    /* If display own source checked, then disable the file choose */
    if(displayOwnSource){
        [searchRootField setEnabled: FALSE];
        [chooseRootButton setEnabled: FALSE];
        //disable other
    }else{
        [searchRootField setEnabled: TRUE];
        [chooseRootButton setEnabled: TRUE];
    }
    
    /* set the font color well */
    color = [NSColor colorWithCalibratedRed:fontColorRed green:fontColorGreen blue:fontColorBlue alpha:fontColorAlpha];
    [colorWell setColor:color];
    
}

- (IBAction)changePrefs:(id)sender {
    NSColor *color;
    
    /* get speed from slider */
    speed = [speedSlider floatValue];

  
    /* get fontColor */
    color = [colorWell color];
    [color getRed:&fontColorRed green:&fontColorGreen blue:&fontColorBlue alpha:&fontColorAlpha ];
    //NSLog(@"fontColor red=%.2f green=%.2f blue=%.2f alpha=%.2f ",fontColorRed, fontColorGreen, fontColorBlue, fontColorAlpha);
    
    /* store prefs */
    [self storePrefs];
    
    /* close the sheet */
    //[self closeSheet:self];
    [[self window] makeFirstResponder:[self window]];
    [NSApp endSheet:[self window] returnCode:NSOKButton];
} 

-(void)storePrefs {
    /* Save preferences */
    /* http://developer.apple.com/techpubs/macosx/AdditionalTechnologies/PreferencePanes/Tasks/Sample.html */
    // NSLog(@"storePrefs");
    // NSLog(@"storePrefs() - fontSize=%i", fontSize);
    
    CFPreferencesSetAppValue((CFStringRef)@"speed", [NSNumber numberWithFloat:speed], (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"fontSize", [NSNumber numberWithFloat:fontSize], (CFStringRef)DOSS);
    // NSLog(@"%@", fontName);
    CFPreferencesSetAppValue((CFStringRef)@"fontName", fontName, (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"searchRoot", theSearchRoot, (CFStringRef)DOSS);
    
    if ( displayOwnSource ){
        // NSLog(@"storePrefs displayOwnSource: TRUE");
        CFPreferencesSetAppValue( (CFStringRef)@"displayOwnSource", kCFBooleanTrue, (CFStringRef)DOSS );
    }else{
        // NSLog(@"storePrefs displayOwnSource: TRUE");
        CFPreferencesSetAppValue( (CFStringRef)@"displayOwnSource",kCFBooleanFalse, (CFStringRef)DOSS );
    }
    
    CFPreferencesSetAppValue((CFStringRef)@"fontColorRed", [NSNumber numberWithFloat:fontColorRed], (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"fontColorBlue", [NSNumber numberWithFloat:fontColorBlue], (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"fontColorGreen", [NSNumber numberWithFloat:fontColorGreen], (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"fontColorAlpha", [NSNumber numberWithFloat:fontColorAlpha], (CFStringRef)DOSS);
    
    CFPreferencesAppSynchronize((CFStringRef)DOSS);
} 

-(void)loadPrefs {
    BOOL initPrefs = NO;
    CFPropertyListRef test;   

    test = CFPreferencesCopyAppValue((CFStringRef) @"speed", (CFStringRef) DOSS);
    speed = [(NSNumber *)test floatValue];
    if (speed < 0 || speed > 30 || test == NULL) {
        initPrefs = YES;
    }
        
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontSize", (CFStringRef) DOSS);
    fontSize = [(NSNumber *)test floatValue];
    //NSLog(@"loadPrefs() - fontSize=%i", fontSize);
    if (fontSize <8 || fontSize > 75 || test == NULL) {
        initPrefs = YES;
    }
    
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontName", (CFStringRef) DOSS);
    fontName = (NSString *)test;
    if (fontName == nil) {
        initPrefs = YES;
    }
        
    test = CFPreferencesCopyAppValue((CFStringRef) @"searchRoot", (CFStringRef) DOSS);
    theSearchRoot = (NSMutableString *)test;
    if (theSearchRoot == nil) {
        initPrefs = YES;
    }
    

        
    test = CFPreferencesCopyAppValue((CFStringRef) @"displayOwnSource", (CFStringRef) DOSS);
    if ( test && CFGetTypeID(test) == CFBooleanGetTypeID() ) {
        if(CFBooleanGetValue(test)){
            displayOwnSource = TRUE;
            // NSLog(@"loadPrefs displayOwnSource: TRUE");
        }else{
            displayOwnSource = FALSE;
            // NSLog(@"loadPrefs displayOwnSource: False");
        }
    }else{
        initPrefs = YES;
    }
    
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorRed", (CFStringRef) DOSS);
    fontColorRed = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorBlue", (CFStringRef) DOSS);
    fontColorBlue = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorGreen", (CFStringRef) DOSS);
    fontColorGreen = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorAlpha", (CFStringRef) DOSS);
    fontColorAlpha = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
        
    /* If values aren't set or are out of range, recreate */
    if (initPrefs) {
        theSearchRoot = @"";
        displayOwnSource = TRUE;
        speed = 15.0;
        fontSize = 25;
        fontColorRed = 0.54;
        fontColorGreen = 1.00;
        fontColorBlue = 0.20;
        fontColorAlpha = 1.00;
        [self storePrefs];
    }
    return;       
}

- (IBAction)chooseRoot:(id)sender {
    NSOpenPanel *op = [NSOpenPanel openPanel];
    NSString *str;
    
    [[self window] makeFirstResponder:[self window]];
    
    [op setAllowsMultipleSelection:NO];
    [op setCanChooseDirectories:YES];
    [op setCanChooseFiles:NO];
    [op setTreatsFilePackagesAsDirectories:NO];
    if ([op runModal]){
        [theSearchRoot release];
        
        /* get the path from the panel */
        str = [op filename];
        
        /* add a / to the end of the path */
        theSearchRoot = [ [NSString alloc] initWithString:[str stringByAppendingString:@"/"] ];        
        
        /* display the new path in the text field */
        [searchRootField setStringValue:theSearchRoot];
        // NSLog(@"theSearchRoot %@", theSearchRoot);
    }
}


- (void)chooseRootOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if ( returnCode == NSOKButton ) {
        NSString *filename = [[sheet filenames] objectAtIndex:0];
        [searchRootField setStringValue:filename];
    }
}


- (IBAction)ownSource:(id)sender {
    displayOwnSource = [sender state];
    if(displayOwnSource){
        [searchRootField setEnabled: FALSE];
        [chooseRootButton setEnabled: FALSE];
        //disable other
    }else{
        [searchRootField setEnabled: TRUE];
        [chooseRootButton setEnabled: TRUE];
    }
}

- (IBAction)setFont:(id)sender {
    [[self window] makeFirstResponder:[self window]];
    (void)[NSFontPanel sharedFontPanel];
    [[NSFontManager sharedFontManager] setSelectedFont:font isMultiple:NO];
    [[NSFontManager sharedFontManager] orderFrontFontPanel:self];
}

- (void)changeFont:(id)sender {
    NSFont *newFont = [sender convertFont:font];
    //NSLog(@"new font desc %@", [newFont description] );
        [fontField setStringValue:[NSString stringWithFormat:@"%@ - %.1f", [newFont displayName], 
        [newFont pointSize]]];
    [fontName release];
    fontName= [[NSString alloc] initWithString:[newFont fontName]];
    fontSize = [newFont pointSize];
    //NSLog(@"changeFont() - fontName=%@ fontSize=%i", fontName, fontSize);
    
}

@end
